@@ -1,13 +1,4 @@
-lazy_static::lazy_static! {
-    static ref COSMOS_ACCOUNT: String = std::env::var("COSMOS_ACCOUNT").unwrap();
-    static ref COSMOS_DATABASE: String = std::env::var("COSMOS_DATABASE").unwrap();
-    static ref COSMOS_MASTER_KEY: String = std::env::var("COSMOS_MASTER_KEY").unwrap();
-    static ref STORAGE_ACCOUNT: String = std::env::var("STORAGE_ACCOUNT").unwrap();
-    static ref STORAGE_MASTER_KEY: String = std::env::var("STORAGE_MASTER_KEY").unwrap();
-    static ref IMAGES_STORAGE_CONTAINER: String = std::env::var("IMAGES_STORAGE_CONTAINER").unwrap();
-    static ref VIDEOS_STORAGE_CONTAINER: String = std::env::var("VIDEOS_STORAGE_CONTAINER").unwrap();
-}
-
+use once_cell::sync::OnceCell;
 use azure_core::{prelude::IfMatchCondition, HttpClient};
 use azure_cosmos::prelude::*;
 use azure_storage::clients::*;
@@ -17,6 +8,25 @@ use std::sync::Arc;
 use tokio::time::{sleep, Duration};
 use uuid::Uuid;
 use warp::{filters::multipart::FormData, Buf as OtherBuf};
+pub use azure_cosmos::prelude::PartitionKeys;
+
+/// This struct keeps the static state of cosmos DB, it must be initialized before any cosmos
+/// functions are called.
+#[derive(Debug)]
+pub struct CosmosState {
+    pub cosmos_account: String,
+    pub cosmos_database: String,
+    pub cosmos_master_key: String,
+    pub storage_account: Option<String>,
+    pub storage_master_key: Option<String>,
+    pub image_storage_container: Option<String>,
+}
+
+static COSMOS_STATE: OnceCell<CosmosState> = OnceCell::new();
+
+pub fn set_state(state: CosmosState) {
+    COSMOS_STATE.set(state).unwrap();
+}
 
 type CosmosError = CosmosErrorStruct;
 
@@ -111,14 +121,17 @@ async fn insert_internal<D: Serialize, P: Into<PartitionKeys>, C: ToString>(
     etag: Option<&str>,
     upsert: bool,
 ) -> Result<String, CosmosError> {
+    let state = COSMOS_STATE.get().ok_or_else(|| {
+        new_cosmos_error(format!("Cosmos state not initialized"))
+    })?;
     let collection_name = collection_name.to_string();
-    let authorization_token = AuthorizationToken::primary_from_base64(&COSMOS_MASTER_KEY)
+    let authorization_token = AuthorizationToken::primary_from_base64(&state.cosmos_master_key)
         .map_err(into_cosmos_error("Could not get authorization token"))?;
 
     let pk = pk.into();
     let http_client: Arc<Box<dyn HttpClient>> = Arc::new(Box::new(reqwest::Client::new()));
-    let client = CosmosClient::new(http_client, COSMOS_ACCOUNT.to_string(), authorization_token);
-    let database_client = client.into_database_client(&*COSMOS_DATABASE);
+    let client = CosmosClient::new(http_client, state.cosmos_account.to_string(), authorization_token);
+    let database_client = client.into_database_client(&state.cosmos_database);
     let collection_client = database_client.into_collection_client(collection_name);
 
     let c = collection_client
@@ -174,15 +187,18 @@ pub async fn get<D: DeserializeOwned, P: Into<PartitionKeys>, C: ToString, S: To
     pk: P,
     document_id: S,
 ) -> Result<(D, String), CosmosError> {
+    let state = COSMOS_STATE.get().ok_or_else(|| {
+        new_cosmos_error(format!("Cosmos state not initialized"))
+    })?;
     let collection_name = collection_name.to_string();
     let document_id = document_id.to_string();
-    let authorization_token = AuthorizationToken::primary_from_base64(&COSMOS_MASTER_KEY)
+    let authorization_token = AuthorizationToken::primary_from_base64(&state.cosmos_master_key)
         .map_err(into_cosmos_error("Could not get authorization token"))?;
 
     let pk = pk.into();
     let http_client: Arc<Box<dyn HttpClient>> = Arc::new(Box::new(reqwest::Client::new()));
-    let client = CosmosClient::new(http_client, COSMOS_ACCOUNT.to_string(), authorization_token);
-    let database_client = client.into_database_client(&*COSMOS_DATABASE);
+    let client = CosmosClient::new(http_client, state.cosmos_account.to_string(), authorization_token);
+    let database_client = client.into_database_client(&state.cosmos_database);
     let collection_client = database_client.into_collection_client(collection_name);
     let document_client = collection_client
         .clone()
@@ -224,15 +240,18 @@ pub async fn modify_async_get_old<
     document_id: S,
     transform: F,
 ) -> Result<(D, D, String), CosmosError> {
+    let state = COSMOS_STATE.get().ok_or_else(|| {
+        new_cosmos_error(format!("Cosmos state not initialized"))
+    })?;
     let collection_name = collection_name.to_string();
     let document_id = document_id.to_string();
-    let authorization_token = AuthorizationToken::primary_from_base64(&COSMOS_MASTER_KEY)
+    let authorization_token = AuthorizationToken::primary_from_base64(&state.cosmos_master_key)
         .map_err(into_cosmos_error("Could not get authorization token"))?;
 
     let pk = pk.into();
     let http_client: Arc<Box<dyn HttpClient>> = Arc::new(Box::new(reqwest::Client::new()));
-    let client = CosmosClient::new(http_client, COSMOS_ACCOUNT.to_string(), authorization_token);
-    let database_client = client.into_database_client(&*COSMOS_DATABASE);
+    let client = CosmosClient::new(http_client, state.cosmos_account.to_string(), authorization_token);
+    let database_client = client.into_database_client(&state.cosmos_database);
     let collection_client = database_client.into_collection_client(collection_name);
     let document_client = collection_client
         .clone()
@@ -315,15 +334,18 @@ pub async fn modify_async<
     document_id: S,
     transform: F,
 ) -> Result<D, CosmosError> {
+    let state = COSMOS_STATE.get().ok_or_else(|| {
+        new_cosmos_error(format!("Cosmos state not initialized"))
+    })?;
     let collection_name = collection_name.to_string();
     let document_id = document_id.to_string();
-    let authorization_token = AuthorizationToken::primary_from_base64(&COSMOS_MASTER_KEY)
+    let authorization_token = AuthorizationToken::primary_from_base64(&state.cosmos_master_key)
         .map_err(into_cosmos_error("Could not get authorization token"))?;
 
     let pk = pk.into();
     let http_client: Arc<Box<dyn HttpClient>> = Arc::new(Box::new(reqwest::Client::new()));
-    let client = CosmosClient::new(http_client, COSMOS_ACCOUNT.to_string(), authorization_token);
-    let database_client = client.into_database_client(&*COSMOS_DATABASE);
+    let client = CosmosClient::new(http_client, state.cosmos_account.to_string(), authorization_token);
+    let database_client = client.into_database_client(&state.cosmos_database);
     let collection_client = database_client.into_collection_client(collection_name);
     let document_client = collection_client
         .clone()
@@ -416,15 +438,18 @@ pub async fn maybe_modify<
     document_id: S,
     transform: F,
 ) -> Result<ModifyReturn<D>, CosmosError> {
+    let state = COSMOS_STATE.get().ok_or_else(|| {
+        new_cosmos_error(format!("Cosmos state not initialized"))
+    })?;
     let collection_name = collection_name.to_string();
     let document_id = document_id.to_string();
-    let authorization_token = AuthorizationToken::primary_from_base64(&COSMOS_MASTER_KEY)
+    let authorization_token = AuthorizationToken::primary_from_base64(&state.cosmos_master_key)
         .map_err(into_cosmos_error("Could not get authorization token"))?;
 
     let pk = pk.into();
     let http_client: Arc<Box<dyn HttpClient>> = Arc::new(Box::new(reqwest::Client::new()));
-    let client = CosmosClient::new(http_client, COSMOS_ACCOUNT.to_string(), authorization_token);
-    let database_client = client.into_database_client(&*COSMOS_DATABASE);
+    let client = CosmosClient::new(http_client, state.cosmos_account.to_string(), authorization_token);
+    let database_client = client.into_database_client(&state.cosmos_database);
     let collection_client = database_client.into_collection_client(collection_name);
     let document_client = collection_client
         .clone()
@@ -505,15 +530,18 @@ pub async fn modify_no_retry<
     document_id: S,
     transform: F,
 ) -> Result<D, CosmosError> {
+    let state = COSMOS_STATE.get().ok_or_else(|| {
+        new_cosmos_error(format!("Cosmos state not initialized"))
+    })?;
     let collection_name = collection_name.to_string();
     let document_id = document_id.to_string();
-    let authorization_token = AuthorizationToken::primary_from_base64(&COSMOS_MASTER_KEY)
+    let authorization_token = AuthorizationToken::primary_from_base64(&state.cosmos_master_key)
         .map_err(into_cosmos_error("Could not get authorization token"))?;
 
     let pk = pk.into();
     let http_client: Arc<Box<dyn HttpClient>> = Arc::new(Box::new(reqwest::Client::new()));
-    let client = CosmosClient::new(http_client, COSMOS_ACCOUNT.to_string(), authorization_token);
-    let database_client = client.into_database_client(&*COSMOS_DATABASE);
+    let client = CosmosClient::new(http_client, state.cosmos_account.to_string(), authorization_token);
+    let database_client = client.into_database_client(&state.cosmos_database);
     let collection_client = database_client.into_collection_client(collection_name);
     let document_client = collection_client
         .clone()
@@ -581,15 +609,18 @@ pub async fn delete<C: ToString, S: ToString, P: Into<PartitionKeys>>(
     document_id: S,
     etag: Option<String>,
 ) -> Result<(), CosmosError> {
+    let state = COSMOS_STATE.get().ok_or_else(|| {
+        new_cosmos_error(format!("Cosmos state not initialized"))
+    })?;
     let collection_name = collection_name.to_string();
     let document_id = document_id.to_string();
-    let authorization_token = AuthorizationToken::primary_from_base64(&COSMOS_MASTER_KEY)
+    let authorization_token = AuthorizationToken::primary_from_base64(&state.cosmos_master_key)
         .map_err(into_cosmos_error("Could not get authorization token"))?;
 
     let pk = pk.into();
     let http_client: Arc<Box<dyn HttpClient>> = Arc::new(Box::new(reqwest::Client::new()));
-    let client = CosmosClient::new(http_client, COSMOS_ACCOUNT.to_string(), authorization_token);
-    let database_client = client.into_database_client(&*COSMOS_DATABASE);
+    let client = CosmosClient::new(http_client, state.cosmos_account.to_string(), authorization_token);
+    let database_client = client.into_database_client(&state.cosmos_database);
     let collection_client = database_client.into_collection_client(collection_name);
     let document_client = collection_client
         .clone()
@@ -617,13 +648,16 @@ pub async fn query_crosspartition_etag<D: DeserializeOwned, P: Into<PartitionKey
     max_count: i32,
     cross_partition: bool,
 ) -> Result<Vec<(D, String)>, CosmosError> {
+    let state = COSMOS_STATE.get().ok_or_else(|| {
+        new_cosmos_error(format!("Cosmos state not initialized"))
+    })?;
     let collection_name = collection_name.to_string();
-    let authorization_token = AuthorizationToken::primary_from_base64(&COSMOS_MASTER_KEY)
+    let authorization_token = AuthorizationToken::primary_from_base64(&state.cosmos_master_key)
         .map_err(into_cosmos_error("Could not get authorization token."))?;
     let pk: PartitionKeys = pk.into();
     let http_client: Arc<Box<dyn HttpClient>> = Arc::new(Box::new(reqwest::Client::new()));
-    let client = CosmosClient::new(http_client, COSMOS_ACCOUNT.to_string(), authorization_token);
-    let database_client = client.into_database_client(&*COSMOS_DATABASE);
+    let client = CosmosClient::new(http_client, state.cosmos_account.to_string(), authorization_token);
+    let database_client = client.into_database_client(&state.cosmos_database);
     let collection_client = database_client.into_collection_client(collection_name.clone());
     let mut documents: Vec<(D, String)> = vec![];
     let mut continuation_token = String::from("");
@@ -697,9 +731,48 @@ pub async fn query<D: DeserializeOwned, P: Into<PartitionKeys>, C: ToString>(
     query_crosspartition(collection_name, pk, query, max_count, false).await
 }
 
+/// Gets the data from a blob storage
+pub async fn get_blob<S: AsRef<str>>(id: S, storage_container: &str) -> Result<Vec<u8>, CosmosError> {
+    let state = COSMOS_STATE.get().ok_or_else(|| {
+        new_cosmos_error(format!("Cosmos state not initialized"))
+    })?;
+    if state.storage_account.is_none() || state.storage_master_key.is_none() {
+        return Err(new_cosmos_error(format!("Cosmos storage values are not initialized")));
+    }
+    let http_client: Arc<Box<dyn HttpClient>> = Arc::new(Box::new(reqwest::Client::new()));
+    let blob_client = StorageAccountClient::new_access_key(
+        http_client,
+        state.storage_account.as_ref().unwrap(),
+        state.storage_master_key.as_ref().unwrap(),
+    )
+    .as_storage_client()
+    .as_container_client(storage_container)
+    .as_blob_client(id.as_ref());
+
+    match blob_client.get().execute().await {
+        Ok(r) => Ok(r.data),
+        Err(err) => {
+            return Err(new_cosmos_error_kind(
+                format!("Could not get blob from storage account {:?}", err),
+                CosmosErrorKind::BlobError,
+            ));
+        }
+    }
+}
 
 /// Uploads form data to the blob storage and returns the blob id
-pub async fn upload_blob(mut f: FormData, blob_type: &str, expected_content_type: &str, storage_container: &str) -> Result<String, CosmosError> {
+pub async fn upload_blob(
+    mut f: FormData,
+    blob_type: &str,
+    expected_content_type: &str,
+    storage_container: &str,
+) -> Result<String, CosmosError> {
+    let state = COSMOS_STATE.get().ok_or_else(|| {
+        new_cosmos_error(format!("Cosmos state not initialized"))
+    })?;
+    if state.storage_account.is_none() || state.storage_master_key.is_none() {
+        return Err(new_cosmos_error(format!("Cosmos storage values are not initialized")));
+    }
     while let Some(r) = f.next().await {
         match r {
             Ok(part) => {
@@ -728,11 +801,11 @@ pub async fn upload_blob(mut f: FormData, blob_type: &str, expected_content_type
                                                 }
                                                 Err(err) => {
                                                     return Err(new_cosmos_error_kind(
-                                                            format!(
-                                                                "Error getting {} data: {:?}",
-                                                                blob_type, err
-                                                            ),
-                                                            CosmosErrorKind::BlobError,
+                                                        format!(
+                                                            "Error getting {} data: {:?}",
+                                                            blob_type, err
+                                                        ),
+                                                        CosmosErrorKind::BlobError,
                                                     ));
                                                 }
                                             }
@@ -747,12 +820,12 @@ pub async fn upload_blob(mut f: FormData, blob_type: &str, expected_content_type
 
                                         let blob_client = StorageAccountClient::new_access_key(
                                             http_client.clone(),
-                                            STORAGE_ACCOUNT.to_string(),
-                                            STORAGE_MASTER_KEY.to_string(),
+                                            state.storage_account.as_ref().unwrap(),
+                                            state.storage_master_key.as_ref().unwrap(),
                                         )
-                                            .as_storage_client()
-                                            .as_container_client(storage_container)
-                                            .as_blob_client(&blob_id);
+                                        .as_storage_client()
+                                        .as_container_client(storage_container)
+                                        .as_blob_client(&blob_id);
 
                                         // Helps preventing spurious data to be uploaded.
                                         let digest = md5::compute(&buf[..]).into();
@@ -763,36 +836,39 @@ pub async fn upload_blob(mut f: FormData, blob_type: &str, expected_content_type
                                                 .hash(&digest)
                                                 .execute()
                                                 .await
-                                                {
-                                                    Ok(r) => r,
-                                                    Err(err) => {
-                                                        return Err(new_cosmos_error_kind(
+                                            {
+                                                Ok(r) => r,
+                                                Err(err) => {
+                                                    return Err(new_cosmos_error_kind(
                                                                 format!("Could not add blob to storage account {:?}", err),
                                                                 CosmosErrorKind::BlobError));
-                                                    }
-                                                };
+                                                }
+                                            };
                                         }
 
                                         // Set blob id.
                                         return Ok(blob_id);
                                     } else {
                                         return Err(new_cosmos_error_kind(
-                                                "Could not get filename for blob.",
-                                                CosmosErrorKind::BlobError,
+                                            "Could not get filename for blob.",
+                                            CosmosErrorKind::BlobError,
                                         ));
                                     }
                                 }
                                 None => {
                                     return Err(new_cosmos_error_kind(
-                                            "Could not get filename for blob.",
-                                            CosmosErrorKind::BlobError,
+                                        "Could not get filename for blob.",
+                                        CosmosErrorKind::BlobError,
                                     ));
                                 }
                             }
                         } else {
                             return Err(new_cosmos_error_kind(
-                                    format!("Blob does not have a {} content-type.", expected_content_type),
-                                    CosmosErrorKind::BlobError,
+                                format!(
+                                    "Blob does not have a {} content-type.",
+                                    expected_content_type
+                                ),
+                                CosmosErrorKind::BlobError,
                             ));
                         }
                     }
@@ -800,26 +876,27 @@ pub async fn upload_blob(mut f: FormData, blob_type: &str, expected_content_type
             }
             Err(err) => {
                 return Err(new_cosmos_error_kind(
-                        format!("Error getting multipart data {:?}", err),
-                        CosmosErrorKind::BlobError,
+                    format!("Error getting multipart data {:?}", err),
+                    CosmosErrorKind::BlobError,
                 ));
             }
         }
     }
     return Err(new_cosmos_error_kind(
-            "No blob provided",
-            CosmosErrorKind::BlobError,
+        "No blob provided",
+        CosmosErrorKind::BlobError,
     ));
 }
 
 /// Uploads a new form data image to the blob storage and returns the image_id
 pub async fn upload_image(f: FormData) -> Result<String, CosmosError> {
-    upload_blob(f, "image", "image", &*IMAGES_STORAGE_CONTAINER).await
-}
-
-/// Uploads a new form data video to the blob storage and returns the video_id
-pub async fn upload_video(f: FormData) -> Result<String, CosmosError> {
-    upload_blob(f, "video", "video", &*VIDEOS_STORAGE_CONTAINER).await
+    let state = COSMOS_STATE.get().ok_or_else(|| {
+        new_cosmos_error(format!("Cosmos state not initialized"))
+    })?;
+    if state.image_storage_container.is_none() {
+        return Err(new_cosmos_error(format!("Cosmos image storage container is not initialized")));
+    }
+    upload_blob(f, "image", "image", &state.image_storage_container.as_ref().unwrap()).await
 }
 
 pub struct CosmosSaga {
